@@ -1,4 +1,4 @@
-# Copyright (c) 2020, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -25,20 +25,23 @@
 import logging, re
 from collections import OrderedDict
 
-from django.db.models import F
+from django.db import transaction
+from django.db.models import F, Max
 from django.http import Http404
 from django.shortcuts import get_object_or_404
-from extra_views.contrib.mixins import SearchableListMixin
-from rest_framework import generics
+from rest_framework import generics, response as http
+from rest_framework.mixins import CreateModelMixin
+from rest_framework.filters import SearchFilter
 from rest_framework.pagination import PageNumberPagination
-from rest_framework import response as http
 
 from ..compat import reverse
-from ..mixins import MatrixMixin
-from ..models import Answer, Matrix, EditableFilter
+from ..mixins import AccountMixin, MatrixMixin
+from ..models import (Answer, Matrix, EditableFilter,
+    EditableFilterEnumeratedAccounts)
 from ..utils import (get_account_model, get_account_serializer,
     get_question_serializer)
-from .serializers import EditableFilterSerializer, MatrixSerializer
+from .serializers import (AccountsFilterAddSerializer,
+    EditableFilterSerializer, MatrixSerializer)
 
 
 LOGGER = logging.getLogger(__name__)
@@ -46,54 +49,17 @@ LOGGER = logging.getLogger(__name__)
 
 class MatrixCreateAPIView(generics.ListCreateAPIView):
     """
-    Filtered list of ``Question``.
+    Lists matrices
 
     **Examples**:
 
     .. code-block:: http
 
-        GET /api/matrix/
+        GET /api/matrix/ HTTP/1.1
 
-        Response:
+    responds
 
-        {
-           "slug": "all",
-           "title": "All accounts against all questions",
-           "metric": {
-               "slug": "all-questions",
-               "title": "All questions",
-               "predicates": []
-           },
-           "cohorts": [{
-               "slug": "all-accounts",
-               "title": "All accounts",
-               "predicates": []
-           }]
-        }
-
-
-    .. code-block:: http
-
-        POST /api/matrix/
-
-        {
-           "slug": "all",
-           "title": "All accounts against all questions",
-           "metric": {
-               "slug": "all-questions",
-               "title": "All questions",
-               "predicates": []
-           },
-           "cohorts": [{
-               "slug": "all-accounts",
-               "title": "All accounts",
-               "predicates": []
-           }]
-        }
-
-        Response:
-
-        201 CREATED
+    .. code-block:: json
 
         {
            "slug": "all",
@@ -115,25 +81,76 @@ class MatrixCreateAPIView(generics.ListCreateAPIView):
     def get_queryset(self):
         return Matrix.objects.all()
 
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a new matrix
+
+        .. code-block:: http
+
+            POST /api/matrix/ HTTP/1.1
+
+        .. code-block:: json
+
+            {
+              "slug": "all",
+              "title": "All accounts against all questions",
+              "metric": {
+                "slug": "all-questions",
+                "title": "All questions",
+                "predicates": []
+              },
+              "cohorts": [{
+                "slug": "all-accounts",
+                "title": "All accounts",
+                "predicates": []
+              }]
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+              "slug": "all",
+              "title": "All accounts against all questions",
+              "metric": {
+                "slug": "all-questions",
+                "title": "All questions",
+                "predicates": []
+              },
+              "cohorts": [{
+                "slug": "all-accounts",
+                "title": "All accounts",
+                "predicates": []
+              }]
+            }
+        """
+        #pylint:disable=useless-super-delegation
+        return super(MatrixCreateAPIView, self).post(request, *args, **kwargs)
+
 
 class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
     """
-    A table of scores for cohorts aganist a metric.
+    Retrieves a matrix of scores for cohorts aganist a metric.
 
-    **Examples**:
+    **Tags**: reporting
+
+    **Examples**
 
     .. code-block:: http
 
-        GET /api/matrix/languages
+        GET /api/matrix/languages HTTP/1.1
 
-        Response:
+   responds
+
+    .. code-block:: json
 
         [{
            "slug": "languages",
            "title": "All cohorts for all questions"
-           "scores":{
-               "portfolio-a": "0.1",
-               "portfolio-b": "0.5",
+           "values": {
+               "portfolio-a": 0.1,
+               "portfolio-b": 0.5
            }
         }]
     """
@@ -143,6 +160,58 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
     lookup_url_kwarg = 'path'
     question_model = get_question_serializer().Meta.model
 
+    def put(self, request, *args, **kwargs):
+        """
+        Updates a matrix of scores for cohorts against a metric
+
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+            PUT /api/matrix/languages HTTP/1.1
+
+        .. code-block:: json
+
+            {
+              "title": "Average scores by supplier industry sub-sector",
+              "cohorts": []
+            }
+
+        responds
+
+        .. code-block:: json
+
+            [{
+              "slug": "languages",
+              "title": "All cohorts for all questions"
+              "values":{
+                "portfolio-a": "0.1",
+                 "portfolio-b": "0.5",
+             }
+            }]
+        """
+        #pylint:disable=useless-super-delegation
+        return super(MatrixDetailAPIView, self).put(
+            request, *args, **kwargs)
+
+    def delete(self, request, *args, **kwargs):
+        """
+        Deletes a matrix of scores for cohorts against a metric
+
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+            DELETE /api/matrix/languages HTTP/1.1
+        """
+        #pylint:disable=useless-super-delegation
+        return super(MatrixDetailAPIView, self).put(
+            request, *args, **kwargs)
+
     def aggregate_scores(self, metric, cohorts, cut=None, accounts=None):
         #pylint:disable=unused-argument,too-many-locals
         if accounts is None:
@@ -246,7 +315,8 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
             'slug': metric.slug,
             'title': metric.title,
             'metric': EditableFilterSerializer().to_representation(metric),
-            'cut': EditableFilterSerializer().to_representation(cut),
+            'cut': (EditableFilterSerializer().to_representation(cut)
+                if cut else None),
             'cohorts': cohort_serializer(many=True).to_representation(cohorts)}
 
         # In some case, a metric and cohort have a connection
@@ -278,18 +348,18 @@ class EditableFilterQuerysetMixin(object):
         return EditableFilter.objects.all()
 
 
-class EditableFilterListAPIView(SearchableListMixin,
-                EditableFilterQuerysetMixin, generics.ListCreateAPIView):
+class EditableFilterListAPIView(EditableFilterQuerysetMixin,
+                                generics.ListAPIView):
     """
-    List fitlers
+    Lists fitlers
 
-    **Tags**: survey
+    **Tags**: reporting
 
     **Examples**
 
     .. code-block:: http
 
-         GET /api/xia/matrix/filters/ HTTP/1.1
+         GET /api/xia/matrix/filters HTTP/1.1
 
     responds
 
@@ -297,159 +367,126 @@ class EditableFilterListAPIView(SearchableListMixin,
 
         {
             "count": 2,
-            previous: null,
-            next: null,
-            results: [
+            "previous": null,
+            "next": null,
+            "results": [
                 {
-                    "slug": "all",
-                    "title": "All",
-                    "tags": "",
+                    "slug": "construction",
+                    "title": "Construction",
+                    "tags": "cohort",
                     "predicates": [
-                        "rank": 1,
-                        "operator": "",
-                        "operand": "",
-                        "field": "",
-                        "selector": ""
+                    {
+                        "rank": 0,
+                        "operator": "contains",
+                        "operand": "construction",
+                        "field": "extra",
+                        "selector": "keepmatching"
+                    }
                     ],
-                    "likely_metric": ""
+                    "likely_metric": null
                 },
                 {
-                    "slug": "none",
-                    "title": "None",
-                    "tags": "",
+                    "slug": "boxes-and-enclosures",
+                    "title": "Boxes and enclosures",
+                    "tags": "metric",
                     "predicates": [
-                        "rank": 1,
-                        "operator": "",
-                        "operand": "",
-                        "field": "",
-                        "selector": ""
-                    ],
-                    "likely_metric": ""
+                    {
+                        "rank": 0,
+                        "operator": "startsWith",
+                        "operand": "/metal/boxes-and-enclosures/",
+                        "field": "path",
+                        "selector": "keepmatching"
+                     }
+                     ],
+                     "likely_metric": null
                 }
             ]
         }
     """
-    search_fields = ['tags']
     serializer_class = EditableFilterSerializer
-
-    def post(self, request, *args, **kwargs):
-        """
-        Create a fitler
-
-        **Tags**: survey
-
-        **Examples**
-
-        .. code-block:: http
-
-             POST /api/xia/matrix/filters/ HTTP/1.1
-
-        responds
-
-        .. code-block:: json
-
-            {
-                "count": 2,
-                previous: null,
-                next: null,
-                results: [
-                    {
-                        "slug": "all",
-                        "title": "All",
-                        "tags": "",
-                        "predicates": [
-                            "rank": 1,
-                            "operator": "",
-                            "operand": "",
-                            "field": "",
-                            "selector": ""
-                        ],
-                        "likely_metric": ""
-                    },
-                    {
-                        "slug": "none",
-                        "title": "None",
-                        "tags": "",
-                        "predicates": [
-                            "rank": 1,
-                            "operator": "",
-                            "operand": "",
-                            "field": "",
-                            "selector": ""
-                        ],
-                        "likely_metric": ""
-                    }
-                ]
-            }
-        """
-        #pylint:disable=useless-super-delegation
-        return super(EditableFilterListAPIView, self).post(
-            request, *args, **kwargs)
+    search_fields = (
+        'tags',
+    )
+    filter_backends = (SearchFilter,)
 
 
-class EditableFilterDetailAPIView(generics.RetrieveUpdateDestroyAPIView):
+class EditableFilterDetailAPIView(AccountMixin,
+                                  generics.RetrieveUpdateDestroyAPIView):
     """
-    Retrieve a fitler
+    Retrieves a fitler
 
-    **Tags**: survey
+    **Tags**: reporting
 
     **Examples**
 
     .. code-block:: http
 
-         GET /api/xia/matrix/filters/all/ HTTP/1.1
+         GET /api/energy-utility/matrix/filters/suppliers HTTP/1.1
 
     responds
 
     .. code-block:: json
 
         {
-            "slug": "all",
-            "title": "All",
-            "tags": "",
-            "predicates": [
+            "slug": "suppliers",
+            "title": "Energy utility suppliers",
+            "tags": "cohort, aggregate",
+            "predicates": [{
                 "rank": 1,
-                "operator": "",
-                "operand": "",
-                "field": "",
-                "selector": ""
-            ],
-            "likely_metric": ""
+                "operator": "contains",
+                "operand": "Energy",
+                "field": "extra",
+                "selector": "keepmatching"
+            }],
+            "likely_metric": null
         }
     """
     serializer_class = EditableFilterSerializer
     lookup_field = 'slug'
     lookup_url_kwarg = 'editable_filter'
 
-    def get_queryset(self):
-        return EditableFilter.objects.all()
+    @property
+    def editable_filter(self):
+        if not hasattr(self, '_editable_filter'):
+            self._editable_filter = get_object_or_404(
+                EditableFilter.objects.all(),
+                account=self.account,
+                slug=self.kwargs.get(self.lookup_url_kwarg))
+        return self._editable_filter
+
+    def get_object(self):
+        editable_filter = self.editable_filter
+        editable_filter.results = get_account_model().objects.filter(
+            filters__editable_filter=self.editable_filter).order_by(
+                'filters__rank')
+        return editable_filter
 
     def put(self, request, *args, **kwargs):
         """
         Updates a fitler
 
-        **Tags**: survey
+        **Tags**: reporting
 
         **Examples**
 
         .. code-block:: http
 
-             PUT /api/xia/matrix/filters/all/ HTTP/1.1
+             PUT /api/energy-utility/matrix/filters/suppliers HTTP/1.1
 
         .. code-block:: json
 
             {
-                "slug": "all",
-                "title": "All",
-                "tags": "",
-                "predicates": [
+                "slug": "suppliers",
+                "title": "Energy utility suppliers",
+                "tags": "cohort, aggregate",
+                "predicates": [{
                     "rank": 1,
-                    "operator": "",
-                    "operand": "",
-                    "field": "",
-                    "selector": ""
-                ],
-                "likely_metric": ""
+                    "operator": "contains",
+                    "operand": "Energy",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }],
+                "likely_metric": null
             }
 
         responds
@@ -457,17 +494,17 @@ class EditableFilterDetailAPIView(generics.RetrieveUpdateDestroyAPIView):
         .. code-block:: json
 
             {
-                "slug": "all",
-                "title": "All",
-                "tags": "",
-                "predicates": [
+                "slug": "suppliers",
+                "title": "Energy utility suppliers",
+                "tags": "cohort, aggregate",
+                "predicates": [{
                     "rank": 1,
-                    "operator": "",
-                    "operand": "",
-                    "field": "",
-                    "selector": ""
-                ],
-                "likely_metric": ""
+                    "operator": "contains",
+                    "operand": "Energy",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }],
+                "likely_metric": null
             }
         """
         #pylint:disable=useless-super-delegation
@@ -478,19 +515,167 @@ class EditableFilterDetailAPIView(generics.RetrieveUpdateDestroyAPIView):
         """
         Deletes a fitler
 
-        **Tags**: survey
+        **Tags**: reporting
 
         **Examples**
 
         .. code-block:: http
 
-             DELETE /api/xia/matrix/filters/all/ HTTP/1.1
+             DELETE /api/energy-utility/matrix/filters/suppliers HTTP/1.1
         """
         #pylint:disable=useless-super-delegation
         return super(EditableFilterDetailAPIView, self).delete(
             request, *args, **kwargs)
 
 
+class AccountsFilterDetailAPIView(CreateModelMixin,
+                                  EditableFilterDetailAPIView):
+    """
+    Retrieves a fitler
+
+    **Tags**: reporting
+
+    **Examples**
+
+    .. code-block:: http
+
+         GET /api/energy-utility/filters/accounts/suppliers HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        vm.items.results = [{
+            facility: "Main factory",
+            fuel_type: "natural-gas",
+            allocation: "PG&E",
+            created_at: null,
+            ends_at: null,
+            amount: 100,
+            unit: "mmbtu"
+        }
+
+        {
+            "slug": "suppliers",
+            "title": "Energy utility suppliers",
+            "tags": "cohort, aggregate",
+            "predicates": [{
+                "rank": 1,
+                "operator": "contains",
+                "operand": "Energy",
+                "field": "extra",
+                "selector": "keepmatching"
+            }],
+            "likely_metric": null
+        }
+    """
+    def get_serializer_class(self):
+        if self.request.method.lower() == 'post':
+            return AccountsFilterAddSerializer
+        return super(AccountsFilterDetailAPIView, self).get_serializer_class()
+
+    def post(self, request, *args, **kwargs):
+        """
+        Updates a fitler
+
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+             PUT /api/energy-utility/matrix/filters/suppliers HTTP/1.1
+
+        .. code-block:: json
+
+            {
+                "slug": "suppliers",
+                "title": "Energy utility suppliers",
+                "tags": "cohort, aggregate",
+                "predicates": [{
+                    "rank": 1,
+                    "operator": "contains",
+                    "operand": "Energy",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }],
+                "likely_metric": null
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+                "slug": "suppliers",
+                "title": "Energy utility suppliers",
+                "tags": "cohort, aggregate",
+                "predicates": [{
+                    "rank": 1,
+                    "operator": "contains",
+                    "operand": "Energy",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }],
+                "likely_metric": null
+            }
+        """
+        #pylint:disable=useless-super-delegation
+        return self.create(request, *args, **kwargs)
+
+    def perform_create(self, serializer):
+        with transaction.atomic():
+            account_slug = serializer.validated_data.get('slug')
+            extra = serializer.validated_data.get('extra')
+            if account_slug:
+                account = get_object_or_404(get_account_model().objects.all(),
+                    slug=account_slug)
+            else:
+                account = get_account_model().objects.create(extra=extra)
+                #XXXaccount.save()
+            last_rank = EditableFilterEnumeratedAccounts.objects.filter(
+                editable_filter=self.editable_filter).aggregate(
+                Max('rank')).get('rank__max')
+            if not last_rank:
+                last_rank = 0
+            EditableFilterEnumeratedAccounts.objects.create(
+                account=account,
+                editable_filter=self.editable_filter,
+                rank=last_rank + 1)
+
+
+class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
+    """
+    Retrieves a fitler
+
+    **Tags**: reporting
+
+    **Examples**
+
+    .. code-block:: http
+
+         GET /api/energy-utility/filters/questions/suppliers HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "slug": "suppliers",
+            "title": "Energy utility suppliers",
+            "tags": "cohort, aggregate",
+            "predicates": [{
+                "rank": 1,
+                "operator": "contains",
+                "operand": "Energy",
+                "field": "extra",
+                "selector": "keepmatching"
+            }],
+            "likely_metric": null
+        }
+    """
+
+
 class EditableFilterPagination(PageNumberPagination):
 
     def paginate_queryset(self, queryset, request, view=None):
@@ -509,17 +694,17 @@ class EditableFilterPagination(PageNumberPagination):
         ]))
 
 
-class EditableFilterObjectsAPIView(generics.ListAPIView):
+class EditableFilterObjectsAPIView(generics.ListCreateAPIView):
     """
     List filter objects
 
-    **Tags**: survey
+    **Tags**: reporting
 
     **Examples**
 
     .. code-block:: http
 
-         GET /api/xia/matrix/filters/ HTTP/1.1
+         GET /api/xia/matrix/filters HTTP/1.1
 
     responds
 
@@ -545,54 +730,119 @@ class EditableFilterObjectsAPIView(generics.ListAPIView):
         return super(EditableFilterObjectsAPIView, self).get(
             request, *args, **kwargs)
 
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a fitler
 
-class AccountListAPIView(EditableFilterObjectsAPIView):
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+             POST /api/xia/matrix/filters HTTP/1.1
+
+        .. code-block:: json
+
+            {
+                "slug": "construction",
+                "title": "Construction",
+                "tags": "cohort",
+                "predicates": [
+                {
+                    "rank": 0,
+                    "operator": "contains",
+                    "operand": "construction",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }
+                ],
+                "likely_metric": null
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+                "slug": "construction",
+                "title": "Construction",
+                "tags": "cohort",
+                "predicates": [
+                {
+                    "rank": 0,
+                    "operator": "contains",
+                    "operand": "construction",
+                    "field": "extra",
+                    "selector": "keepmatching"
+                }
+                ],
+                "likely_metric": null
+            }
+        """
+        #pylint:disable=useless-super-delegation
+        return super(EditableFilterObjectsAPIView, self).post(
+            request, *args, **kwargs)
+
+
+class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
     """
     Filtered list of ``EditableFilter``.
 
+    **Tags**: reporting
+
     **Examples**:
 
     .. code-block:: http
 
-        GET /api/questions/languages
+        GET /api/supplier-1/matrix/accounts HTTP/1.1
 
-        Response:
+    responds
+
+    .. code-block:: json
 
         {
-           "slug": "languages",
-           "title": "All questions related to languages"
-           "predicates":[{
-               "operator": "contains",
-               "operand": "language",
-               "field": "text",
-               "selector":"keepmatching"
+            "count": 1,
+            "previous": null,
+            "next": null,
+            "results": [
+            {
+               "slug": "supplier-1",
+               "full_name": "Supplier 1",
+               "email": "supplier-1@localhost.localdomain"
            }]
         }
     """
     serializer_class = get_account_serializer()
 
 
-class QuestionListAPIView(EditableFilterObjectsAPIView):
+class QuestionsFilterListAPIView(EditableFilterObjectsAPIView):
     """
     Filtered list of ``Question``.
 
+    **Tags**: reporting
+
     **Examples**:
 
     .. code-block:: http
 
-        GET /api/questions/languages
+        GET /api/supplier-1/matrix/questions HTTP/1.1
 
-        Response:
+    responds
+
+    .. code-block:: json
 
         {
-           "slug": "languages",
-           "title": "All questions related to languages"
-           "predicates":[{
+            "count": 1,
+            "previous": null,
+            "next": null,
+            "results": [
+            {
                "operator": "contains",
                "operand": "language",
                "field": "text",
                "selector":"keepmatching"
-           }]
+            }]
         }
     """
     serializer_class = get_question_serializer()
